# -*- coding: utf-8 -*-

import math

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from qgis.core import *

class TestPlugin:
  def __init__(self, iface):
    self.iface = iface

  def initGui(self):
    self.actionRun = QAction("Test renderer", self.iface.mainWindow())

    self.iface.addPluginToMenu("Test renderer", self.actionRun)
    self.iface.addToolBarIcon(self.actionRun)

    self.actionRun.triggered.connect(self.run)

  def unload(self):
    self.iface.removeToolBarIcon(self.actionRun)
    self.iface.removePluginMenu("Test renderer", self.actionRun)

  def run(self):
    canvas = self.iface.mapCanvas()
    layers = []
    for layer in canvas.layers():
      layers.append(unicode(layer.id()))

    myRed = QgsProject.instance().readNumEntry("Gui", "/CanvasColorRedPart", 255)[0]
    myGreen = QgsProject.instance().readNumEntry("Gui", "/CanvasColorGreenPart", 255)[0]
    myBlue = QgsProject.instance().readNumEntry("Gui", "/CanvasColorBluePart", 255)[0]
    color = QColor(myRed, myGreen, myBlue)

    image = QImage(256, 256, QImage.Format_ARGB32_Premultiplied)

    projector = QgsCoordinateTransform(QgsCoordinateReferenceSystem("EPSG:4326"),
                                       QgsCoordinateReferenceSystem("EPSG:3395")
                                      )

    scaleCalc = QgsScaleCalculator()
    scaleCalc.setDpi(image.logicalDpiX())
    scaleCalc.setMapUnits(QgsCoordinateReferenceSystem("EPSG:3395").mapUnits())

    labeling = QgsPalLabeling()
    renderer = QgsMapRenderer()
    renderer.setOutputSize(image.size(), image.logicalDpiX())
    renderer.setDestinationCrs(QgsCoordinateReferenceSystem("EPSG:3395"))
    renderer.setProjectionsEnabled(True)
    renderer.setLabelingEngine(labeling)
    renderer.setLayerSet(layers)

    painter = QPainter()

    extent = QgsRectangle(82.6171875000000000,54.9776136706962575,82.9687500000000000,55.1788676632819914)

    for i in xrange(400000):
      renderer.setExtent(projector.transform(extent))
      scale = scaleCalc.calculate(renderer.extent(), 256)
      renderer.setScale(scale)
      image.fill(color)
      painter.begin(image)
      renderer.render(painter)
      painter.end()
